[metadata]
name = {package}
version = {version}
author = {author}
author_email = {email}
description = {description}
long_description = file: README.md
long_description_content_type = text/markdown
classifiers =
    Programming Language :: Python :: 3
    
[options]
python_requires = >={py_min}, <{py_max}
package_dir =
    =src
packages =
    {package}